"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CredentialsModule = void 0;
const tsyringe_1 = require("tsyringe");
const AgentConfig_1 = require("../../agent/AgentConfig");
const MessageSender_1 = require("../../agent/MessageSender");
const helpers_1 = require("../../agent/helpers");
const ServiceDecorator_1 = require("../../decorators/service/ServiceDecorator");
const error_1 = require("../../error");
const storage_1 = require("../../storage");
const DidCommMessageRepository_1 = require("../../storage/didcomm/DidCommMessageRepository");
const did_1 = require("../../utils/did");
const services_1 = require("../connections/services");
const dids_1 = require("../dids");
const routing_1 = require("../routing");
const CredentialProtocolVersion_1 = require("./CredentialProtocolVersion");
const CredentialState_1 = require("./CredentialState");
const V1CredentialService_1 = require("./protocol/v1/V1CredentialService");
const V2CredentialService_1 = require("./protocol/v2/V2CredentialService");
const CredentialRepository_1 = require("./repository/CredentialRepository");
let CredentialsModule = class CredentialsModule {
    // note some of the parameters passed in here are temporary, as we intend
    // to eventually remove CredentialsModule
    constructor(messageSender, connectionService, agentConfig, credentialRepository, mediationRecipientService, didCommMessageRepository, v1Service, v2Service, didResolver) {
        this.messageSender = messageSender;
        this.connectionService = connectionService;
        this.credentialRepository = credentialRepository;
        this.agentConfig = agentConfig;
        this.mediatorRecipientService = mediationRecipientService;
        this.didCommMessageRepo = didCommMessageRepository;
        this.logger = agentConfig.logger;
        this.v1Service = v1Service;
        this.v2Service = v2Service;
        this.didResolver = didResolver;
        this.serviceMap = {
            [CredentialProtocolVersion_1.CredentialProtocolVersion.V1]: this.v1Service,
            [CredentialProtocolVersion_1.CredentialProtocolVersion.V2]: this.v2Service,
        };
        this.logger.debug(`Initializing Credentials Module for agent ${this.agentConfig.label}`);
    }
    getService(protocolVersion) {
        return this.serviceMap[protocolVersion];
    }
    async declineOffer(credentialRecordId) {
        const credentialRecord = await this.getById(credentialRecordId);
        credentialRecord.assertState(CredentialState_1.CredentialState.OfferReceived);
        // with version we can get the Service
        const service = this.getService(credentialRecord.protocolVersion);
        await service.updateState(credentialRecord, CredentialState_1.CredentialState.Declined);
        return credentialRecord;
    }
    async negotiateOffer(options) {
        if (!options.credentialRecordId) {
            throw new error_1.AriesFrameworkError(`No credential record id found in negotiateCredentialOffer`);
        }
        const credentialRecord = await this.getById(options.credentialRecordId);
        const version = credentialRecord.protocolVersion;
        const service = this.getService(version);
        const { message } = await service.negotiateOffer(options, credentialRecord);
        if (!credentialRecord.connectionId) {
            throw new error_1.AriesFrameworkError(`No connection id for credential record ${credentialRecord.id} not found. Connection-less issuance does not support negotiation`);
        }
        const connection = await this.connectionService.getById(credentialRecord.connectionId);
        const outboundMessage = helpers_1.createOutboundMessage(connection, message);
        await this.messageSender.sendMessage(outboundMessage);
        return credentialRecord;
    }
    /**
     * Initiate a new credential exchange as holder by sending a credential proposal message
     * to the connection with the specified credential options
     *
     * @param options configuration to use for the proposal
     * @returns Credential exchange record associated with the sent proposal message
     */
    async proposeCredential(options) {
        // get the version
        const version = options.protocolVersion;
        // with version we can get the Service
        if (!version) {
            throw new error_1.AriesFrameworkError('Missing Protocol Version');
        }
        const service = this.getService(version);
        this.logger.debug(`Got a CredentialService object for version ${version}`);
        const connection = await this.connectionService.getById(options.connectionId);
        // will get back a credential record -> map to Credential Exchange Record
        const { credentialRecord, message } = await service.createProposal(options);
        this.logger.debug('We have a message (sending outbound): ', message);
        // send the message here
        const outbound = helpers_1.createOutboundMessage(connection, message);
        this.logger.debug('In proposeCredential: Send Proposal to Issuer');
        await this.messageSender.sendMessage(outbound);
        return credentialRecord;
    }
    /**
     * Accept a credential proposal as issuer (by sending a credential offer message) to the connection
     * associated with the credential record.
     *
     * @param options config object for the proposal (and subsequent offer) which replaces previous named parameters
     * @returns Credential exchange record associated with the credential offer
     *
     */
    async acceptProposal(options) {
        const credentialRecord = await this.getById(options.credentialRecordId);
        if (!credentialRecord.connectionId) {
            throw new error_1.AriesFrameworkError('Missing connection id in v2 acceptCredentialProposal');
        }
        const version = credentialRecord.protocolVersion;
        // with version we can get the Service
        const service = this.getService(version);
        // will get back a credential record -> map to Credential Exchange Record
        const { message } = await service.acceptProposal(options, credentialRecord);
        const connection = await this.connectionService.getById(credentialRecord.connectionId);
        this.logger.debug('We have an offer message (sending outbound): ', message);
        // send the message here
        const outbound = helpers_1.createOutboundMessage(connection, message);
        this.logger.debug('In acceptCredentialProposal: Send Proposal to Issuer');
        await this.messageSender.sendMessage(outbound);
        return credentialRecord;
    }
    /**
     * Accept a credential offer as holder (by sending a credential request message) to the connection
     * associated with the credential record.
     *
     * @param options The object containing config options of the offer to be accepted
     * @returns Object containing offer associated credential record
     */
    async acceptOffer(options) {
        const record = await this.getById(options.credentialRecordId);
        const service = this.getService(record.protocolVersion);
        this.logger.debug(`Got a CredentialService object for this version; version = ${service.getVersion()}`);
        const offerMessage = await service.getOfferMessage(record.id);
        // Use connection if present
        if (record.connectionId) {
            const connection = await this.connectionService.getById(record.connectionId);
            if (!connection.did) {
                throw new error_1.AriesFrameworkError(`Connection record ${connection.id} has no 'did'`);
            }
            const didDocument = await this.didResolver.resolveDidDocument(connection.did);
            const verificationMethod = await dids_1.findVerificationMethodByKeyType('Ed25519VerificationKey2018', didDocument);
            if (!verificationMethod) {
                throw new error_1.AriesFrameworkError('Invalid DidDocument: Missing verification method with type Ed25519VerificationKey2018 to use as indy holder did');
            }
            const indyDid = did_1.getIndyDidFromVerificationMethod(verificationMethod);
            const requestOptions = {
                comment: options.comment,
                autoAcceptCredential: options.autoAcceptCredential,
                holderDid: indyDid,
            };
            const { message, credentialRecord } = await service.createRequest(record, requestOptions);
            await this.didCommMessageRepo.saveAgentMessage({
                agentMessage: message,
                role: storage_1.DidCommMessageRole.Sender,
                associatedRecordId: credentialRecord.id,
            });
            this.logger.debug('We have sent a credential request');
            const outboundMessage = helpers_1.createOutboundMessage(connection, message);
            this.logger.debug('We have a proposal message (sending outbound): ', message);
            await this.messageSender.sendMessage(outboundMessage);
            await this.credentialRepository.update(credentialRecord);
            return credentialRecord;
        }
        // Use ~service decorator otherwise
        else if (offerMessage === null || offerMessage === void 0 ? void 0 : offerMessage.service) {
            // Create ~service decorator
            const routing = await this.mediatorRecipientService.getRouting();
            const ourService = new ServiceDecorator_1.ServiceDecorator({
                serviceEndpoint: routing.endpoints[0],
                recipientKeys: [routing.recipientKey.publicKeyBase58],
                routingKeys: routing.routingKeys.map((key) => key.publicKeyBase58),
            });
            const recipientService = offerMessage.service;
            const requestOptions = {
                comment: options.comment,
                autoAcceptCredential: options.autoAcceptCredential,
                holderDid: ourService.recipientKeys[0],
            };
            const { message, credentialRecord } = await service.createRequest(record, requestOptions);
            // Set and save ~service decorator to record (to remember our verkey)
            message.service = ourService;
            await this.didCommMessageRepo.saveAgentMessage({
                agentMessage: message,
                role: storage_1.DidCommMessageRole.Sender,
                associatedRecordId: credentialRecord.id,
            });
            await this.credentialRepository.update(credentialRecord);
            await this.messageSender.sendMessageToService({
                message,
                service: recipientService.resolvedDidCommService,
                senderKey: ourService.resolvedDidCommService.recipientKeys[0],
                returnRoute: true,
            });
            return credentialRecord;
        }
        // Cannot send message without connectionId or ~service decorator
        else {
            throw new error_1.AriesFrameworkError(`Cannot accept offer for credential record without connectionId or ~service decorator on credential offer.`);
        }
    }
    /**
     * Negotiate a credential proposal as issuer (by sending a credential offer message) to the connection
     * associated with the credential record.
     *
     * @param options configuration for the offer see {@link NegotiateProposalOptions}
     * @returns Credential exchange record associated with the credential offer
     *
     */
    async negotiateProposal(options) {
        const credentialRecord = await this.getById(options.credentialRecordId);
        // get the version
        const version = credentialRecord.protocolVersion;
        // with version we can get the Service
        const service = this.getService(version);
        const { message } = await service.negotiateProposal(options, credentialRecord);
        if (!credentialRecord.connectionId) {
            throw new error_1.AriesFrameworkError(`No connection id for credential record ${credentialRecord.id} not found. Connection-less issuance does not support negotiation`);
        }
        const connection = await this.connectionService.getById(credentialRecord.connectionId);
        // use record connection id to get the connection
        const outboundMessage = helpers_1.createOutboundMessage(connection, message);
        await this.messageSender.sendMessage(outboundMessage);
        return credentialRecord;
    }
    /**
     * Initiate a new credential exchange as issuer by sending a credential offer message
     * to the connection with the specified connection id.
     *
     * @param options config options for the credential offer
     * @returns Credential exchange record associated with the sent credential offer message
     */
    async offerCredential(options) {
        if (!options.connectionId) {
            throw new error_1.AriesFrameworkError('Missing connectionId on offerCredential');
        }
        if (!options.protocolVersion) {
            throw new error_1.AriesFrameworkError('Missing protocol version in offerCredential');
        }
        const connection = await this.connectionService.getById(options.connectionId);
        const service = this.getService(options.protocolVersion);
        this.logger.debug(`Got a CredentialService object for version ${options.protocolVersion}`);
        // pass the connection in rather than leave it to the service layer to get the connection
        const { message, credentialRecord } = await service.createOffer(Object.assign(Object.assign({}, options), { connection }));
        this.logger.debug('Offer Message successfully created; message= ', message);
        const outboundMessage = helpers_1.createOutboundMessage(connection, message);
        await this.messageSender.sendMessage(outboundMessage);
        return credentialRecord;
    }
    /**
     * Accept a credential request as holder (by sending a credential request message) to the connection
     * associated with the credential record.
     *
     * @param options The object containing config options of the request
     * @returns CredentialExchangeRecord updated with information pertaining to this request
     */
    async acceptRequest(options) {
        if (!options.credentialRecordId) {
            throw new error_1.AriesFrameworkError('Missing credential record id in acceptRequest');
        }
        const record = await this.getById(options.credentialRecordId);
        // with version we can get the Service
        const service = this.getService(record.protocolVersion);
        this.logger.debug(`Got a CredentialService object for version ${record.protocolVersion}`);
        const { message, credentialRecord } = await service.createCredential(record, options);
        this.logger.debug('We have a credential message (sending outbound): ', message);
        const requestMessage = await service.getRequestMessage(credentialRecord.id);
        const offerMessage = await service.getOfferMessage(credentialRecord.id);
        // Use connection if present
        if (credentialRecord.connectionId) {
            const connection = await this.connectionService.getById(credentialRecord.connectionId);
            const outboundMessage = helpers_1.createOutboundMessage(connection, message);
            await this.messageSender.sendMessage(outboundMessage);
        }
        // Use ~service decorator otherwise
        else if ((requestMessage === null || requestMessage === void 0 ? void 0 : requestMessage.service) && (offerMessage === null || offerMessage === void 0 ? void 0 : offerMessage.service)) {
            const recipientService = requestMessage.service;
            const ourService = offerMessage.service;
            message.service = ourService;
            await this.credentialRepository.update(credentialRecord);
            await this.messageSender.sendMessageToService({
                message,
                service: recipientService.resolvedDidCommService,
                senderKey: ourService.resolvedDidCommService.recipientKeys[0],
                returnRoute: true,
            });
        }
        // Cannot send message without connectionId or ~service decorator
        else {
            throw new error_1.AriesFrameworkError(`Cannot accept request for credential record without connectionId or ~service decorator on credential offer / request.`);
        }
        await this.didCommMessageRepo.saveAgentMessage({
            agentMessage: message,
            role: storage_1.DidCommMessageRole.Sender,
            associatedRecordId: credentialRecord.id,
        });
        return credentialRecord;
    }
    /**
     * Accept a credential as holder (by sending a credential acknowledgement message) to the connection
     * associated with the credential record.
     *
     * @param credentialRecordId The id of the credential record for which to accept the credential
     * @returns credential exchange record associated with the sent credential acknowledgement message
     *
     */
    async acceptCredential(credentialRecordId) {
        const record = await this.getById(credentialRecordId);
        // with version we can get the Service
        const service = this.getService(record.protocolVersion);
        this.logger.debug(`Got a CredentialService object for version ${record.protocolVersion}`);
        const { message, credentialRecord } = await service.createAck(record);
        const requestMessage = await service.getRequestMessage(credentialRecord.id);
        const credentialMessage = await service.getCredentialMessage(credentialRecord.id);
        if (credentialRecord.connectionId) {
            const connection = await this.connectionService.getById(credentialRecord.connectionId);
            const outboundMessage = helpers_1.createOutboundMessage(connection, message);
            await this.messageSender.sendMessage(outboundMessage);
        }
        // Use ~service decorator otherwise
        else if ((credentialMessage === null || credentialMessage === void 0 ? void 0 : credentialMessage.service) && (requestMessage === null || requestMessage === void 0 ? void 0 : requestMessage.service)) {
            const recipientService = credentialMessage.service;
            const ourService = requestMessage.service;
            await this.messageSender.sendMessageToService({
                message,
                service: recipientService.resolvedDidCommService,
                senderKey: ourService.resolvedDidCommService.recipientKeys[0],
                returnRoute: true,
            });
        }
        // Cannot send message without connectionId or ~service decorator
        else {
            throw new error_1.AriesFrameworkError(`Cannot accept credential without connectionId or ~service decorator on credential message.`);
        }
        return credentialRecord;
    }
    /**
     * Retrieve a credential record by id
     *
     * @param credentialRecordId The credential record id
     * @throws {RecordNotFoundError} If no record is found
     * @return The credential record
     *
     */
    getById(credentialRecordId) {
        return this.credentialRepository.getById(credentialRecordId);
    }
    /**
     * Retrieve all credential records
     *
     * @returns List containing all credential records
     */
    getAll() {
        return this.credentialRepository.getAll();
    }
    /**
     * Find a credential record by id
     *
     * @param credentialRecordId the credential record id
     * @returns The credential record or null if not found
     */
    findById(credentialRecordId) {
        return this.credentialRepository.findById(credentialRecordId);
    }
    /**
     * Delete a credential record by id, also calls service to delete from wallet
     *
     * @param credentialId the credential record id
     * @param options the delete credential options for the delete operation
     */
    async deleteById(credentialId, options) {
        const credentialRecord = await this.getById(credentialId);
        const service = this.getService(credentialRecord.protocolVersion);
        return service.delete(credentialRecord, options);
    }
    /**
     * Initiate a new credential exchange as issuer by creating a credential offer
     * not bound to any connection. The offer must be delivered out-of-band to the holder
     * @param options The credential options to use for the offer
     * @returns The credential record and credential offer message
     */
    async createOffer(options) {
        const service = this.getService(options.protocolVersion);
        this.logger.debug(`Got a CredentialService object for version ${options.protocolVersion}`);
        const { message, credentialRecord } = await service.createOffer(options);
        this.logger.debug('Offer Message successfully created; message= ', message);
        return { message, credentialRecord };
    }
};
CredentialsModule = __decorate([
    tsyringe_1.scoped(tsyringe_1.Lifecycle.ContainerScoped),
    __metadata("design:paramtypes", [MessageSender_1.MessageSender,
        services_1.ConnectionService,
        AgentConfig_1.AgentConfig,
        CredentialRepository_1.CredentialRepository,
        routing_1.MediationRecipientService,
        DidCommMessageRepository_1.DidCommMessageRepository,
        V1CredentialService_1.V1CredentialService,
        V2CredentialService_1.V2CredentialService,
        dids_1.DidResolverService])
], CredentialsModule);
exports.CredentialsModule = CredentialsModule;
//# sourceMappingURL=CredentialsModule.js.map