"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CredentialMessageBuilder = void 0;
const AriesFrameworkError_1 = require("../../../../error/AriesFrameworkError");
const uuid_1 = require("../../../../utils/uuid");
const CredentialProtocolVersion_1 = require("../../CredentialProtocolVersion");
const CredentialState_1 = require("../../CredentialState");
const CredentialExchangeRecord_1 = require("../../repository/CredentialExchangeRecord");
const V2CredentialPreview_1 = require("./V2CredentialPreview");
const V2IssueCredentialMessage_1 = require("./messages/V2IssueCredentialMessage");
const V2OfferCredentialMessage_1 = require("./messages/V2OfferCredentialMessage");
const V2ProposeCredentialMessage_1 = require("./messages/V2ProposeCredentialMessage");
const V2RequestCredentialMessage_1 = require("./messages/V2RequestCredentialMessage");
class CredentialMessageBuilder {
    /**
     * Create a v2 credential proposal message according to the logic contained in the format service. The format services
     * contain specific logic related to indy, jsonld etc. with others to come.
     *
     * @param formats {@link CredentialFormatService} array of format service objects each containing format-specific logic
     * @param proposal {@link ProposeCredentialOptions} object containing (optionally) the linked attachments
     * @param _threadId optional thread id for this message service
     * @return a version 2.0 credential propose message see {@link V2ProposeCredentialMessage}
     */
    async createProposal(formatServices, proposal) {
        if (formatServices.length === 0)
            throw new AriesFrameworkError_1.AriesFrameworkError('no format services provided to createProposal');
        // create message
        // there are two arrays in each message, one for formats the other for attachments
        const formats = [];
        const filterAttachments = [];
        let credentialPreview;
        for (const formatService of formatServices) {
            const { format, attachment, preview } = await formatService.createProposal(proposal);
            credentialPreview !== null && credentialPreview !== void 0 ? credentialPreview : (credentialPreview = preview);
            filterAttachments.push(attachment);
            formats.push(format);
        }
        const options = {
            id: this.generateId(),
            formats,
            filtersAttach: filterAttachments,
            comment: proposal.comment,
            credentialProposal: credentialPreview,
        };
        const message = new V2ProposeCredentialMessage_1.V2ProposeCredentialMessage(options);
        const props = {
            connectionId: proposal.connectionId,
            threadId: message.threadId,
            state: CredentialState_1.CredentialState.ProposalSent,
            autoAcceptCredential: proposal === null || proposal === void 0 ? void 0 : proposal.autoAcceptCredential,
            protocolVersion: CredentialProtocolVersion_1.CredentialProtocolVersion.V2,
            credentials: [],
        };
        // Create the v2 record
        const credentialRecord = new CredentialExchangeRecord_1.CredentialExchangeRecord(props);
        return { message, credentialRecord };
    }
    /**
     * accept a v2 credential proposal message according to the logic contained in the format service. The format services
     * contain specific logic related to indy, jsonld etc. with others to come.
     *
     * @param message {@link V2ProposeCredentialMessage} object containing (optionally) the linked attachments
     * @param connectionId optional connection id for the agent to agent connection
     * @return a version 2.0 credential record object see {@link CredentialRecord}
     */
    processProposal(message, connectionId) {
        var _a;
        const props = {
            connectionId,
            threadId: message.threadId,
            state: CredentialState_1.CredentialState.ProposalReceived,
            credentialAttributes: (_a = message.credentialProposal) === null || _a === void 0 ? void 0 : _a.attributes,
            protocolVersion: CredentialProtocolVersion_1.CredentialProtocolVersion.V2,
            credentials: [],
        };
        return new CredentialExchangeRecord_1.CredentialExchangeRecord(props);
    }
    async createOfferAsResponse(formatServices, credentialRecord, options) {
        if (formatServices.length === 0)
            throw new AriesFrameworkError_1.AriesFrameworkError('no format services provided to createOfferAsResponse');
        // create message
        // there are two arrays in each message, one for formats the other for attachments
        const formats = [];
        const offerAttachments = [];
        const credentialPreview = new V2CredentialPreview_1.V2CredentialPreview({ attributes: [] });
        for (const formatService of formatServices) {
            const { attachment, preview, format } = await formatService.createOffer(options);
            if (preview && preview.attributes.length > 0)
                credentialPreview.attributes = preview.attributes;
            formats.push(format);
            offerAttachments.push(attachment);
            await formatService.processOffer(attachment, credentialRecord);
        }
        const messageProps = {
            id: this.generateId(),
            formats,
            comment: options.comment,
            offerAttachments,
            credentialPreview,
        };
        const credentialOfferMessage = new V2OfferCredentialMessage_1.V2OfferCredentialMessage(messageProps);
        credentialOfferMessage.setThread({ threadId: credentialRecord.threadId });
        credentialRecord.credentialAttributes = credentialPreview.attributes;
        return credentialOfferMessage;
    }
    /**
     * Create a {@link V2RequestCredentialMessage}
     *
     * @param formatService correct service for format, indy, w3c etc.
     * @param record The credential record for which to create the credential request
     * @param offer Additional configuration for the offer if present (might not be for W3C)
     * @returns Object containing request message and associated credential record
     *
     */
    async createRequest(options) {
        var _a;
        if (options.formatServices.length === 0)
            throw new AriesFrameworkError_1.AriesFrameworkError('no format services provided to createRequest');
        const { formatServices, offerMessage, record, requestOptions, holderDid } = options;
        const formats = [];
        const requestAttachments = [];
        for (const formatService of formatServices) {
            // use the attach id in the formats object to find the correct attachment
            const offerAttachment = formatService.getAttachment(offerMessage.formats, offerMessage.messageAttachment);
            if (offerAttachment) {
                requestOptions.offerAttachment = offerAttachment;
            }
            else {
                throw new AriesFrameworkError_1.AriesFrameworkError(`Missing data payload in attachment in credential Record ${record.id}`);
            }
            const { format, attachment } = await formatService.createRequest(requestOptions, record, holderDid);
            requestOptions.requestAttachment = attachment;
            formats.push(format);
            requestAttachments.push(attachment);
        }
        const messageOptions = {
            id: this.generateId(),
            formats: formats,
            requestsAttach: requestAttachments,
            comment: requestOptions.comment,
        };
        const message = new V2RequestCredentialMessage_1.V2RequestCredentialMessage(messageOptions);
        message.setThread(record);
        (_a = record.autoAcceptCredential) !== null && _a !== void 0 ? _a : (record.autoAcceptCredential = requestOptions.autoAcceptCredential);
        return { message, credentialRecord: record };
    }
    /**
     * Create a {@link V2OfferCredentialMessage} as beginning of protocol process.
     *
     * @param formatService {@link CredentialFormatService} the format service object containing format-specific logic
     * @param options attributes of the original offer
     * @returns Object containing offer message and associated credential record
     *
     */
    async createOffer(formatServices, options) {
        if (formatServices.length === 0)
            throw new AriesFrameworkError_1.AriesFrameworkError('no format services provided to createOffer');
        const { autoAcceptCredential, comment, connection } = options;
        const formats = [];
        const offerAttachments = [];
        const credentialPreview = new V2CredentialPreview_1.V2CredentialPreview({ attributes: [] });
        const offerMap = new Map();
        for (const formatService of formatServices) {
            const { attachment, preview, format } = await formatService.createOffer(options);
            if (preview && preview.attributes.length > 0)
                credentialPreview.attributes = preview.attributes;
            offerMap.set(attachment, formatService);
            offerAttachments.push(attachment);
            formats.push(format);
        }
        const messageProps = {
            id: this.generateId(),
            formats,
            comment: comment,
            offerAttachments,
            replacementId: undefined,
            credentialPreview,
        };
        // Construct v2 offer message
        const message = new V2OfferCredentialMessage_1.V2OfferCredentialMessage(messageProps);
        const recordProps = {
            connectionId: connection === null || connection === void 0 ? void 0 : connection.id,
            threadId: message.threadId,
            autoAcceptCredential,
            state: CredentialState_1.CredentialState.OfferSent,
            credentialAttributes: credentialPreview === null || credentialPreview === void 0 ? void 0 : credentialPreview.attributes,
            protocolVersion: CredentialProtocolVersion_1.CredentialProtocolVersion.V2,
            credentials: [],
        };
        const credentialRecord = new CredentialExchangeRecord_1.CredentialExchangeRecord(recordProps);
        for (const offersAttach of offerMap.keys()) {
            const service = offerMap.get(offersAttach);
            // service MUST be defined here as we extract the key of the key-value pair and get the value
            await (service === null || service === void 0 ? void 0 : service.processOffer(offersAttach, credentialRecord));
        }
        return { credentialRecord, message };
    }
    /**
     * Create a {@link V2IssueCredentialMessage} - we issue the credentials to the holder with this message
     *
     * @param formatService {@link CredentialFormatService} the format service object containing format-specific logic
     * @param offerMessage the original offer message
     * @returns Object containing offer message and associated credential record
     *
     */
    async createCredential(credentialFormats, record, serviceOptions, requestMessage, offerMessage) {
        const formats = [];
        const credentialAttachments = [];
        for (const formatService of credentialFormats) {
            const requestAttachment = formatService.getAttachment(requestMessage.formats, requestMessage.messageAttachment);
            if (!requestAttachment)
                throw new Error(`Missing request attachment in createCredential`);
            const offerAttachment = formatService.getAttachment(offerMessage.formats, offerMessage.messageAttachment);
            const { format, attachment } = await formatService.createCredential(serviceOptions, record, requestAttachment, offerAttachment);
            formats.push(format);
            credentialAttachments.push(attachment);
        }
        const messageOptions = {
            id: this.generateId(),
            formats: formats,
            credentialsAttach: credentialAttachments,
            comment: serviceOptions.comment,
        };
        const message = new V2IssueCredentialMessage_1.V2IssueCredentialMessage(messageOptions);
        return { message, credentialRecord: record };
    }
    generateId() {
        return uuid_1.uuid();
    }
}
exports.CredentialMessageBuilder = CredentialMessageBuilder;
//# sourceMappingURL=CredentialMessageBuilder.js.map