import type { AgentMessage } from '../../../../agent/AgentMessage';
import type { HandlerInboundMessage } from '../../../../agent/Handler';
import type { InboundMessageContext } from '../../../../agent/models/InboundMessageContext';
import type { CredentialProtocolMsgReturnType, ServiceOfferCredentialOptions } from '../../CredentialServiceOptions';
import type { AcceptProposalOptions, AcceptRequestOptions, NegotiateOfferOptions, NegotiateProposalOptions, ProposeCredentialOptions, RequestCredentialOptions } from '../../CredentialsModuleOptions';
import type { CredentialFormatService } from '../../formats/CredentialFormatService';
import type { CredentialFormats, CredentialFormatSpec, HandlerAutoAcceptOptions } from '../../formats/models/CredentialFormatServiceOptions';
import { AgentConfig } from '../../../../agent/AgentConfig';
import { Dispatcher } from '../../../../agent/Dispatcher';
import { EventEmitter } from '../../../../agent/EventEmitter';
import { DidCommMessageRepository } from '../../../../storage';
import { ConnectionService } from '../../../connections/services/ConnectionService';
import { DidResolverService } from '../../../dids';
import { MediationRecipientService } from '../../../routing';
import { CredentialProtocolVersion } from '../../CredentialProtocolVersion';
import { CredentialFormatType } from '../../CredentialsModuleOptions';
import { IndyCredentialFormatService } from '../../formats/indy/IndyCredentialFormatService';
import { CredentialRepository, CredentialExchangeRecord } from '../../repository';
import { RevocationService } from '../../services';
import { CredentialService } from '../../services/CredentialService';
import { V2OfferCredentialHandler } from './handlers/V2OfferCredentialHandler';
import { V2ProposeCredentialHandler } from './handlers/V2ProposeCredentialHandler';
import { V2CredentialAckMessage } from './messages/V2CredentialAckMessage';
import { V2IssueCredentialMessage } from './messages/V2IssueCredentialMessage';
import { V2OfferCredentialMessage } from './messages/V2OfferCredentialMessage';
import { V2ProposeCredentialMessage } from './messages/V2ProposeCredentialMessage';
import { V2RequestCredentialMessage } from './messages/V2RequestCredentialMessage';
export declare class V2CredentialService extends CredentialService {
    private connectionService;
    private credentialMessageBuilder;
    private indyCredentialFormatService;
    private serviceFormatMap;
    constructor(connectionService: ConnectionService, credentialRepository: CredentialRepository, eventEmitter: EventEmitter, dispatcher: Dispatcher, agentConfig: AgentConfig, mediationRecipientService: MediationRecipientService, didCommMessageRepository: DidCommMessageRepository, indyCredentialFormatService: IndyCredentialFormatService, revocationService: RevocationService, didResolver: DidResolverService);
    /**
     * Create a {@link V2ProposeCredentialMessage} not bound to an existing credential exchange.
     *
     * @param proposal The ProposeCredentialOptions object containing the important fields for the credential message
     * @returns Object containing proposal message and associated credential record
     *
     */
    createProposal(proposal: ProposeCredentialOptions): Promise<CredentialProtocolMsgReturnType<V2ProposeCredentialMessage>>;
    /**
     * Method called by {@link V2ProposeCredentialHandler} on reception of a propose credential message
     * We do the necessary processing here to accept the proposal and do the state change, emit event etc.
     * @param messageContext the inbound propose credential message
     * @returns credential record appropriate for this incoming message (once accepted)
     */
    processProposal(messageContext: HandlerInboundMessage<V2ProposeCredentialHandler>): Promise<CredentialExchangeRecord>;
    acceptProposal(proposal: AcceptProposalOptions, credentialRecord: CredentialExchangeRecord): Promise<CredentialProtocolMsgReturnType<V2OfferCredentialMessage>>;
    /**
     * Create a {@link AcceptProposalOptions} object used by handler
     *
     * @param credentialRecord {@link CredentialRecord} the record containing the proposal
     * @return options attributes of the proposal
     *
     */
    private createAcceptProposalOptions;
    /**
     * Negotiate a credential proposal as issuer (by sending a credential offer message) to the connection
     * associated with the credential record.
     *
     * @param options configuration for the offer see {@link NegotiateProposalOptions}
     * @returns Credential exchange record associated with the credential offer
     *
     */
    negotiateProposal(options: NegotiateProposalOptions, credentialRecord: CredentialExchangeRecord): Promise<CredentialProtocolMsgReturnType<V2OfferCredentialMessage>>;
    /**
     * Create a {@link ProposePresentationMessage} as response to a received credential offer.
     * To create a proposal not bound to an existing credential exchange, use {@link createProposal}.
     *
     * @param credentialRecord The credential record for which to create the credential proposal
     * @param config Additional configuration to use for the proposal
     * @returns Object containing proposal message and associated credential record
     *
     */
    negotiateOffer(options: NegotiateOfferOptions, credentialRecord: CredentialExchangeRecord): Promise<CredentialProtocolMsgReturnType<V2ProposeCredentialMessage>>;
    /**
     * Create a {@link V2OfferCredentialMessage} as beginning of protocol process. If no connectionId is provided, the
     * exchange will be created without a connection for usage in oob and connection-less issuance.
     *
     * @param formatService {@link CredentialFormatService} the format service object containing format-specific logic
     * @param options attributes of the original offer
     * @returns Object containing offer message and associated credential record
     *
     */
    createOffer(options: ServiceOfferCredentialOptions): Promise<CredentialProtocolMsgReturnType<V2OfferCredentialMessage>>;
    /**
     * Create a {@link OfferCredentialMessage} as response to a received credential proposal.
     * To create an offer not bound to an existing credential exchange, use {@link V2CredentialService#createOffer}.
     *
     * @param credentialRecord The credential record for which to create the credential offer
     * @param credentialTemplate The credential template to use for the offer
     * @returns Object containing offer message and associated credential record
     *
     */
    createOfferAsResponse(credentialRecord: CredentialExchangeRecord, proposal?: ServiceOfferCredentialOptions | NegotiateProposalOptions): Promise<V2OfferCredentialMessage>;
    /**
     * Method called by {@link V2OfferCredentialHandler} on reception of a offer credential message
     * We do the necessary processing here to accept the offer and do the state change, emit event etc.
     * @param messageContext the inbound offer credential message
     * @returns credential record appropriate for this incoming message (once accepted)
     */
    processOffer(messageContext: HandlerInboundMessage<V2OfferCredentialHandler>): Promise<CredentialExchangeRecord>;
    /**
     * Create a {@link V2RequestCredentialMessage}
     *
     * @param credentialRecord The credential record for which to create the credential request
     * @param options request options for creating this request
     * @returns Object containing request message and associated credential record
     *
     */
    createRequest(record: CredentialExchangeRecord, options: RequestCredentialOptions, holderDid?: string): Promise<CredentialProtocolMsgReturnType<V2RequestCredentialMessage>>;
    /**
     * Process a received {@link RequestCredentialMessage}. This will not accept the credential request
     * or send a credential. It will only update the existing credential record with
     * the information from the credential request message. Use {@link createCredential}
     * after calling this method to create a credential.
     *
     * @param messageContext The message context containing a v2 credential request message
     * @returns credential record associated with the credential request message
     *
     */
    processRequest(messageContext: InboundMessageContext<V2RequestCredentialMessage>): Promise<CredentialExchangeRecord>;
    /**
     * Create a {@link IssueCredentialMessage} as response to a received credential request.
     *
     * @param credentialRecord The credential record for which to create the credential
     * @param options Additional configuration to use for the credential
     * @returns Object containing issue credential message and associated credential record
     *
     */
    createCredential(record: CredentialExchangeRecord, options: AcceptRequestOptions): Promise<CredentialProtocolMsgReturnType<V2IssueCredentialMessage>>;
    /**
     * Process a received {@link IssueCredentialMessage}. This will not accept the credential
     * or send a credential acknowledgement. It will only update the existing credential record with
     * the information from the issue credential message. Use {@link createAck}
     * after calling this method to create a credential acknowledgement.
     *
     * @param messageContext The message context containing an issue credential message
     *
     * @returns credential record associated with the issue credential message
     *
     */
    processCredential(messageContext: InboundMessageContext<V2IssueCredentialMessage>): Promise<CredentialExchangeRecord>;
    /**
     * Create a {@link V2CredentialAckMessage} as response to a received credential.
     *
     * @param credentialRecord The credential record for which to create the credential acknowledgement
     * @returns Object containing credential acknowledgement message and associated credential record
     *
     */
    createAck(credentialRecord: CredentialExchangeRecord): Promise<CredentialProtocolMsgReturnType<V2CredentialAckMessage>>;
    /**
     * Process a received {@link CredentialAckMessage}.
     *
     * @param messageContext The message context containing a credential acknowledgement message
     * @returns credential record associated with the credential acknowledgement message
     *
     */
    processAck(messageContext: InboundMessageContext<V2CredentialAckMessage>): Promise<CredentialExchangeRecord>;
    /**
     * Register the v2 handlers. These handlers supplement, ie are created in addition to, the existing
     * v1 handlers.
     */
    registerHandlers(): void;
    shouldAutoRespondToProposal(options: HandlerAutoAcceptOptions): Promise<boolean>;
    shouldAutoRespondToOffer(credentialRecord: CredentialExchangeRecord, offerMessage: V2OfferCredentialMessage, proposeMessage?: V2ProposeCredentialMessage): boolean;
    shouldAutoRespondToRequest(credentialRecord: CredentialExchangeRecord, requestMessage: V2RequestCredentialMessage, proposeMessage?: V2ProposeCredentialMessage, offerMessage?: V2OfferCredentialMessage): boolean;
    shouldAutoRespondToCredential(credentialRecord: CredentialExchangeRecord, credentialMessage: V2IssueCredentialMessage): boolean;
    getOfferMessage(id: string): Promise<AgentMessage | null>;
    getRequestMessage(id: string): Promise<AgentMessage | null>;
    getCredentialMessage(id: string): Promise<AgentMessage | null>;
    update(credentialRecord: CredentialExchangeRecord): Promise<void>;
    /**
     * Returns the protocol version for this credential service
     * @returns v2 as this is the v2 service
     */
    getVersion(): CredentialProtocolVersion;
    /**
     * Gets the correct formatting service for this credential record type, eg indy or jsonld. Others may be
     * added in the future.
     * Each formatting service knows how to format the message structure for the specific record type
     * @param credentialFormatType the format type, indy, jsonld, jwt etc.
     * @returns the formatting service.
     */
    getFormatService(credentialFormatType: CredentialFormatType): CredentialFormatService;
    /**
     * Retrieve a credential record by connection id and thread id
     *
     * @param connectionId The connection id
     * @param threadId The thread id
     * @throws {RecordNotFoundError} If no record is found
     * @throws {RecordDuplicateError} If multiple records are found
     * @returns The credential record
     */
    getByThreadAndConnectionId(threadId: string, connectionId?: string): Promise<CredentialExchangeRecord>;
    /**
     * Get all the format service objects for a given credential format from an incoming message
     * @param messageFormats the format objects containing the format name (eg indy)
     * @return the credential format service objects in an array - derived from format object keys
     */
    getFormatsFromMessage(messageFormats: CredentialFormatSpec[]): CredentialFormatService[];
    /**
     * Get all the format service objects for a given credential format
     * @param credentialFormats the format object containing various optional parameters
     * @return the credential format service objects in an array - derived from format object keys
     */
    getFormats(credentialFormats: CredentialFormats): CredentialFormatService[];
}
