"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.V2OfferCredentialHandler = void 0;
const helpers_1 = require("../../../../../agent/helpers");
const ServiceDecorator_1 = require("../../../../../decorators/service/ServiceDecorator");
const AriesFrameworkError_1 = require("../../../../../error/AriesFrameworkError");
const storage_1 = require("../../../../../storage");
const did_1 = require("../../../../../utils/did");
const dids_1 = require("../../../../dids");
const V2OfferCredentialMessage_1 = require("../messages/V2OfferCredentialMessage");
const V2ProposeCredentialMessage_1 = require("../messages/V2ProposeCredentialMessage");
class V2OfferCredentialHandler {
    constructor(credentialService, agentConfig, mediationRecipientService, didCommMessageRepository, didResolver) {
        this.supportedMessages = [V2OfferCredentialMessage_1.V2OfferCredentialMessage];
        this.credentialService = credentialService;
        this.agentConfig = agentConfig;
        this.mediationRecipientService = mediationRecipientService;
        this.didCommMessageRepository = didCommMessageRepository;
        this.didResolver = didResolver;
    }
    async handle(messageContext) {
        const credentialRecord = await this.credentialService.processOffer(messageContext);
        const offerMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: V2OfferCredentialMessage_1.V2OfferCredentialMessage,
        });
        const proposeMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: V2ProposeCredentialMessage_1.V2ProposeCredentialMessage,
        });
        if (!offerMessage) {
            throw new AriesFrameworkError_1.AriesFrameworkError('Missing offer message in V2OfferCredentialHandler');
        }
        const shouldAutoRespond = this.credentialService.shouldAutoRespondToOffer(credentialRecord, offerMessage, proposeMessage !== null && proposeMessage !== void 0 ? proposeMessage : undefined);
        if (shouldAutoRespond) {
            return await this.createRequest(credentialRecord, messageContext, offerMessage);
        }
    }
    async createRequest(record, messageContext, offerMessage) {
        this.agentConfig.logger.info(`Automatically sending request with autoAccept on ${this.agentConfig.autoAcceptCredentials}`);
        if (messageContext.connection) {
            if (!messageContext.connection.did) {
                throw new AriesFrameworkError_1.AriesFrameworkError(`Connection record ${messageContext.connection.id} has no 'did'`);
            }
            const didDocument = await this.didResolver.resolveDidDocument(messageContext.connection.did);
            const verificationMethod = await dids_1.findVerificationMethodByKeyType('Ed25519VerificationKey2018', didDocument);
            if (!verificationMethod) {
                throw new AriesFrameworkError_1.AriesFrameworkError('Invalid DidDocument: Missing verification method with type Ed25519VerificationKey2018 to use as indy holder did');
            }
            const indyDid = did_1.getIndyDidFromVerificationMethod(verificationMethod);
            const { message, credentialRecord } = await this.credentialService.createRequest(record, {
                holderDid: indyDid,
            });
            await this.didCommMessageRepository.saveAgentMessage({
                agentMessage: message,
                role: storage_1.DidCommMessageRole.Receiver,
                associatedRecordId: credentialRecord.id,
            });
            return helpers_1.createOutboundMessage(messageContext.connection, message);
        }
        else if (offerMessage === null || offerMessage === void 0 ? void 0 : offerMessage.service) {
            const routing = await this.mediationRecipientService.getRouting();
            const ourService = new ServiceDecorator_1.ServiceDecorator({
                serviceEndpoint: routing.endpoints[0],
                recipientKeys: [routing.recipientKey.publicKeyBase58],
                routingKeys: routing.routingKeys.map((key) => key.publicKeyBase58),
            });
            const recipientService = offerMessage.service;
            const { message, credentialRecord } = await this.credentialService.createRequest(record, {
                holderDid: ourService.recipientKeys[0],
            });
            // Set and save ~service decorator to record (to remember our verkey)
            message.service = ourService;
            await this.credentialService.update(credentialRecord);
            await this.didCommMessageRepository.saveAgentMessage({
                agentMessage: message,
                role: storage_1.DidCommMessageRole.Sender,
                associatedRecordId: credentialRecord.id,
            });
            return helpers_1.createOutboundServiceMessage({
                payload: message,
                service: recipientService.resolvedDidCommService,
                senderKey: ourService.resolvedDidCommService.recipientKeys[0],
            });
        }
        this.agentConfig.logger.error(`Could not automatically create credential request`);
    }
}
exports.V2OfferCredentialHandler = V2OfferCredentialHandler;
//# sourceMappingURL=V2OfferCredentialHandler.js.map